/*
    MSound.h - Header file for MSound.dll
    by MUTATE Systems 2017
*/

#pragma once

#ifdef MSOUND_EXPORTS
#define MSOUND_API __declspec(dllexport)
#else
#define MSOUND_API __declspec(dllimport)
#endif

// Initialize DirectSound and bind to hwnd
MSOUND_API bool MSoundInitialize(HWND hwnd);

// Shutdown DirectSound.
MSOUND_API void MSoundShutdown(void);

class ISoundPlayer {
public:
    virtual void stop(void) = 0;     // Stop the sound immediately.
	virtual void stopLoop(void) = 0; // Stop the loop.

    virtual void setVolume(LONG) = 0; // Specify by decibel. Please use ratioToDecibel()
	virtual void setPan(LONG) = 0;    // -10000 to 10000. Please see the spec of IDirectSoundBuffer8::SetPan()
};

class ISoundEffect {
public:
    // To start the sound.
	// You can control the sound by returned ISoundPlayer.
	virtual ISoundPlayer* play(bool bLoop=0, LONG vol=0, LONG pan = 0) = 0;

    // Load sound file (wav or ogg)
	// nClone means the count of the sound that can be play concurrently.
    // If bPlayNonActive is true, it continues playing even if the window loses the focus.
	MSOUND_API static ISoundEffect* create(const char* szFileName, int nClone = 1, bool bPlayNonActive = false);

    virtual void release(void) = 0;
};

class IBgmPlayer : public ISoundPlayer {
public:
    // Load and start playing
    virtual bool start(const char* szBgmFileName, bool bLoop = true) = 0;

    // Create a new player
    // If bPlayNonActive is true, it continues playing even if the window loses the focus.
    MSOUND_API static IBgmPlayer* create(bool bPlayNonActive = false);

    virtual void release(void) = 0;
};

// Convert from float(0.0 - 1.0) to decibel
MSOUND_API int ratioToDecibel(float ratio);
