/*
    MSoundDemo.cpp - Sample demo for MSound.dll
    by MUTATE Systems 2017
*/

#pragma comment(lib, "MSound.lib")
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <ObjBase.h>

#include "MSound.h"

IBgmPlayer* g_pBGMPlayer;
ISoundEffect* g_pSE_Test;

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
    case WM_CREATE:
        SetTimer(hWnd, 1, 2000, NULL);
        break;

    case WM_PAINT:
        {
            PAINTSTRUCT ps;
            HDC hdc = BeginPaint(hWnd, &ps);
            const char* sz = "Playing BGM";
            TextOutA(hdc, 10, 10, sz, strlen(sz));
            EndPaint(hWnd, &ps);
        }
		break;

    case WM_TIMER:
        if(g_pSE_Test)
            g_pSE_Test->play(); // Play sound effect
        break;

    case WM_DESTROY:
		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
    // CoInitialize is required before CreateWindow if you use DirectSound.
    CoInitialize(NULL);

    WNDCLASSEX wcex = {0};
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.hInstance		= hInstance;
    wcex.hCursor		= LoadCursor(NULL , IDC_ARROW);
    wcex.hbrBackground	= (HBRUSH)GetStockObject(WHITE_BRUSH);
	wcex.lpszClassName	= L"MSoundDemo";

    RegisterClassEx(&wcex);

    HWND hWnd = CreateWindow(wcex.lpszClassName, L"MSound Demo", WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

    if (!hWnd) return FALSE;

    RECT rc = {0};
    GetWindowRect(hWnd, &rc);
    MoveWindow(hWnd, rc.left, rc.top, 600, 100, FALSE);
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    // Initialize the MSound.dll
    MSoundInitialize(hWnd);

    // Create BGM Player
    g_pBGMPlayer = IBgmPlayer::create();

    if(!g_pBGMPlayer) {
        MessageBoxA(hWnd, "Cannot initialize MSound", "Error", MB_OK);
        return 0;
    }

    // Load and start BGM
    if(!g_pBGMPlayer->start("bgm.ogg")) {
        MessageBoxA(hWnd, "Cannot start BGM", "Error", MB_OK);
        return 0;
    }

    // Load sound effect
    g_pSE_Test = ISoundEffect::create("sht-eb.ogg", 5);

    MSG msg = {0};
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

    g_pSE_Test->release();
    g_pBGMPlayer->release();

    // Shutdown MSound.dll
    MSoundShutdown();

	return (int) msg.wParam;
}
